from pychart import *

OUTPUT_FORMAT = 'png'
CHART_WIDTH = 800
CHART_HEIGHT = 600

theme.use_color = True


def generate(data, n):
    # Theme config
    theme.get_options()
    theme.output_format = OUTPUT_FORMAT
    theme.output_file = 'chart' + str(n) + '.' + OUTPUT_FORMAT

    # Chart defaults
    chart_object.set_defaults(area.T, size=(CHART_WIDTH, CHART_HEIGHT), y_range=(0, 3), x_coord=category_coord.T(data, 0))
    chart_object.set_defaults(bar_plot.T, data=data)

    # Plot
    xaxis = axis.X(label="Hilos, numero de entradas", format=lambda x: str(int(x*2)))  # Because it's really 2x the number of threads
    yaxis = axis.Y(label="Speedup", tic_interval=0.2)

    ar = area.T(legend=legend.T(), x_axis=xaxis, y_axis=yaxis)

    bar_plot.fill_styles.reset()

    plot1 = bar_plot.T(label="16 entries", hcol=1, cluster=(0, 6))
    plot2 = bar_plot.T(label="32 entries", hcol=2, cluster=(1, 6))
    plot3 = bar_plot.T(label="64 entries", hcol=3, cluster=(2, 6))
    plot4 = bar_plot.T(label="128 entries", hcol=4, cluster=(3, 6))
    plot5 = bar_plot.T(label="256 entries", hcol=5, cluster=(4, 6))
    plot6 = bar_plot.T(label="512 entries", hcol=6, cluster=(5, 6))
    ar.add_plot(plot1, plot2, plot3, plot4, plot5, plot6)

    ar.draw()


def generate_fails(data, name):
    # Theme config
    theme.get_options()
    theme.output_format = OUTPUT_FORMAT
    theme.output_file = 'chart' + str(name) + '.' + OUTPUT_FORMAT

    # Chart defaults
    chart_object.set_defaults(area.T, size=(CHART_WIDTH, CHART_HEIGHT), y_range=(0, None), x_coord=category_coord.T(data, 0))
    chart_object.set_defaults(bar_plot.T, data=data)

    # Plot
    xaxis = axis.X(label="Hilos, numero de entradas", format=lambda x: str(int(x*2)))
    yaxis = axis.Y(label="Number of aborted transactions")

    ar = area.T(legend=legend.T(), x_axis=xaxis, y_axis=yaxis)

    bar_plot.fill_styles.reset()

    plot1 = bar_plot.T(label="16 entries", hcol=1, cluster=(0, 6))
    plot2 = bar_plot.T(label="32 entries", hcol=2, cluster=(1, 6))
    plot3 = bar_plot.T(label="64 entries", hcol=3, cluster=(2, 6))
    plot4 = bar_plot.T(label="128 entries", hcol=4, cluster=(3, 6))
    plot5 = bar_plot.T(label="256 entries", hcol=5, cluster=(4, 6))
    plot6 = bar_plot.T(label="512 entries", hcol=6, cluster=(5, 6))

    ar.add_plot(plot1, plot2, plot3, plot4, plot5, plot6)
    ar.draw()


def generate_fails_reasons_multidim(data, name):
    # Theme config
    theme.get_options()
    theme.output_format = OUTPUT_FORMAT
    theme.output_file = 'chart' + str(name) + '.' + OUTPUT_FORMAT

    # Chart defaults
    chart_object.set_defaults(area.T, size=(CHART_WIDTH, CHART_HEIGHT), y_range=(0, None), x_coord=category_coord.T(data, 0))
    chart_object.set_defaults(bar_plot.T, data=data)

    # Plot
    xaxis = axis.X(label="Hilos", format=lambda x: str(int(x*2)))
    yaxis = axis.Y(label="Quantity")

    ar = area.T(legend=legend.T(), x_axis=xaxis, y_axis=yaxis)

    bar_plot.fill_styles.reset()

    capac16 = bar_plot.T(label="Capacity 16 entries", hcol=1, cluster=(0, 6))
    conflict16 = bar_plot.T(label="Conflict 16 entries", hcol=2, stack_on=capac16, cluster=(0, 6))
    capac32 = bar_plot.T(label="Capacity 32 entries", hcol=3, cluster=(1, 6))
    conflict32 = bar_plot.T(label="Conflict 32 entries", hcol=4, stack_on=capac32, cluster=(1, 6))
    capac64 = bar_plot.T(label="Capacity 64 entries", hcol=5, cluster=(2, 6))
    conflict64 = bar_plot.T(label="Conflict 64 entries", hcol=6, stack_on=capac64, cluster=(2, 6))
    capac128 = bar_plot.T(label="Capacity 128 entries", hcol=7, cluster=(3, 6))
    conflict128 = bar_plot.T(label="Conflict 128 entries", hcol=8, stack_on=capac128, cluster=(3, 6))
    capac256 = bar_plot.T(label="Capacity 256 entries", hcol=9, cluster=(4, 6))
    conflict256 = bar_plot.T(label="Conflict 256 entries", hcol=10, stack_on=capac256, cluster=(4, 6))
    capac512 = bar_plot.T(label="Capacity 512 entries", hcol=11, cluster=(5, 6))
    conflict512 = bar_plot.T(label="Conflict 512 entries", hcol=12, stack_on=capac512, cluster=(5, 6))

    ar.add_plot(
        capac16,
        conflict16,
        capac32,
        conflict32,
        capac64,
        conflict64,
        capac128,
        conflict128,
        capac256,
        conflict256,
        capac512,
        conflict512
    )
    ar.draw()


def generate_fails_reasons(data, name):
    # Theme config
    theme.get_options()
    theme.use_color = True
    theme.output_format = OUTPUT_FORMAT
    theme.output_file = 'chart' + str(name) + '.' + OUTPUT_FORMAT

    # Chart defaults
    chart_object.set_defaults(area.T, size=(CHART_WIDTH, CHART_HEIGHT), y_range=(0, None), x_coord=category_coord.T(data, 1))
    chart_object.set_defaults(bar_plot.T, data=data)

    # Plot
    xaxis = axis.X(label="Numero de entradas")
    yaxis = axis.Y(label="Numero de errores")

    ar = area.T(legend=legend.T(), x_axis=xaxis, y_axis=yaxis)

    bar_plot.fill_styles.reset()

    capacity = bar_plot.T(label="Capacity", hcol=3)
    conflict = bar_plot.T(label="Conflict", hcol=4, stack_on=capacity)

    ar.add_plot(capacity, conflict)
    ar.draw()
