# Introducción y objetivos
\pagenumbering{arabic}

En el mundo de la informática es un tema recurrente que, pese a que los
ordenadores continúan duplicando el número de transistores cada dos años [@wgsimon_english:_2011],
cada vez encontramos más posibilidades a las plataformas de computación que
requieren rendimientos exponencialmente mayores.

## El cuello de botella en servidores web
Un claro ejemplo es la evolución de la población en Internet. Se estima que en
el año 2000 existían un total de 400 millones de usuarios en Internet, y en 2015
esa cifra ha subido hasta los 3200 millones, multiplicándose el número de
usuarios por 8 en estos 15 años [@_ict]. Esta inmensa cantidad de usuarios provoca
grandes problemas a los individuales y empresas que proveen los servicios en
red.

Es común hablar de los ataques de denegación de servicio distribuidos (DDoS)
como uno de los problemas más frecuentes para el buen funcionamiento de Internet
a día de hoy, y es un problema real debido a que el rendimiento de los
servidores no puede crecer suficientemente rápido como para acomodar los
inmensos picos que se dan, ya sea voluntariamente y con buena intención [@_slashdot_2016], o
todo lo contrario [@cimpanu_new].

Uno de los cuellos de botella de los servidores web suelen ser las bases de
datos. Además de las restricciones de rendimiento que puedan sufrir por los
accesos a disco, se espera que las bases de datos garanticen las propiedades
ACID (atomicidad, consistencia, aislamiento, durabilidad) [@gray_transaction_1981]
[@haerder_principles_1983] lo que complica el control de la concurrencia.

La solución tradicional a este problema suele ser usar algún tipo de cierre de
exclusión mutua por software. Esta decisión afecta al rendimiento en general
provocando la serialización de muchas operaciones que de otra manera podrían ser
paralelas y con el _overhead_ que conlleva la implementación de estos sistemas;
todo a cambio de mantener las propiedades ACID.

## La perspectiva de memoria transccional
Otro método más moderno para solucionar este problema es la memoria
transaccional. En este caso, es el propio sistema el que se encarga de detectar
estas violaciones y repetir las operaciones invalidadas sin afectar a la validez
de las transacciones. Sin embargo, las implementaciones accesibles a día de hoy
son sobre software y el resultado es que el _overhead_ que implican no compensa
las ventajas a la hora de programar.

Puede que este panorama comience a cambiar con la popularización de la memoria
transaccional sobre hardware. Los procesadores de última generación de Intel
(sexta generación, denominada _Skylake_) destinados a consumidores generales ya
soportan una implementación de memoria transaccional sobre hardware que promete
proporcionar las ventajas de la memoria transaccional sin las graves
penalizaciones en el rendimiento de las implementaciones por software.

## Objetivos
En este trabajo quiero explorar las posibilidades de las extensiones TSX que
acercan la memoria transaccional sobre hardware a todos los consumidores. A
través de una serie de pruebas de rendimiento investigaré bajo qué condiciones
la solución de memoria transaccional por hardware que implementa Intel a partir
de su sexta generación es más eficiente que las implementaciones habituales con
cierres de exclusión mutua.

Una de las conclusiones que se podrán extraer es el estado de esta
implementación y si merece la pena hacer soluciones a medida que la aprovechen a
día de hoy; y qué problemas implica y cómo debería mejorar esta tecnología para
generalizar su uso y ser comparativamente mejor que las alternativas sobre
software.

Esta línea de investigación puede ser útil pues si se demuestra que el rendimiento
es superior al de un cerrojo de exclusión mutua, y que la implementación es tan o más
sencilla, se podrá diseñar sofware que aproveche estas características llevando
el paralelismo real a aplicaciones en las que a día de hoy ni siquiera se considera.

Un objetivo secundario de este trabajo es ver cómo se puede generalizar la paralelización
de forma que se pueda aplicar a diversas estructuras y algoritmos. Por ello, el último
capítulo reune algunas pinceladas sobre el paralelismo en estructuras irregulares y concretamente
en un framework denominado Galois que persigue este objetivo.
