# Aplicaciones paralelas con estructuras compartidas irregulares
Además del caso que se decidió que vertebrara este trabajo, el de un servidor web, se investigaron otras
posibles aplicaciones donde la memoria transaccional pudiera ser de utilidad. Una de ellas es la paralelización
en aplicaciones con estructuras irregulares.

Este tipo de estructuras se caracterizan porque no se corresponden con estructuras fácilmente paralelizables,
como tablas o árboles equilibrados. Las formas que suelen tomar son grafos o conjuntos, con un número variable
de dimensiones, y sin una forma definida. Además, en algunos algoritmos, incluso la forma de la estructura
cambia durante la ejecución.

## Ejemplos de aplicaciones con estructuras irregulares
Uno de los ejemplos más de moda de este tipo de aplicaciones es el análisis de redes sociales. Este problema
suele modelarse con un grafo donde cada vértice es una persona en la red social y cada arista es una relación
entre dos personas. Con esta sencilla premisa se pueden llegar a generar estructuras muy complejas debido al
gran número de datos con los que se suelen realizar estas investigaciones. Además, la estructura puede complicarse
con pesos en las aristas o más información en los vértices. El análisis de estas redes puede usarse para fines
muy diversos, como encontrar el comportamiento de la difusión de información en ellas [@brashears_error_2016] o
analizar su estructura durante eventos concretos [@gonzalez-bailon_networked_2016].

Otro ejemplo es en análisis de estructuras en dos o tres dimensiones para hacerlas más simples con el objetivo de
renderizarlas, imprimirlas, etc. Sin lugar a dudas, hay muchos más ejemplos de este tipo de estructras, y lo que
tienen en común es que es muy costoso paralelizar algoritmos que necesitan trabajar con ellas y que es aún más
complicado crear frameworks que permitan generalizar esta paralelización para poder usarlo en varios problemas y
estructuras irregulares distintas. A continuación, veremos un framework que intenta alcanzar este objetivo.

## Framework Galois
### Introducción
El proyecto Galois, desarrollado por un equipo en la Universidad de Texas en Austin es un sistema que facilita la
paralelización de código en C++ o Java, aunque sus últimas versiones solo están disponibles para C++. El sistema se
enfoca en hacer posible y sencilla la paralelización en computaciones sobre estructuras irregulares, como las que
normalmente se encuentran en problemas que involucran estructuras irregulares de datos enlazados que cambian o no su forma durante
la ejecución. Algunos ejemplos de este tipo de problemas son la triangulación de Delaunay o la generación de árboles
recubridores a partir de un grafo que contengan todos los vértices de éste pero sin ciclos.

Galois consigue este objetivo añadiendo unas nuevas estructuras de datos que ayudan a definir la naturaleza de los datos
que representan de forma que el sistema sepa hasta qué punto y con qué condiciones se puede paralelizar el algoritmo.
Estos metadatos se hacen llegar al motor de Galois, quien se asegura de que la ejecución sea segura y todo lo rápida
que se lo permitan las restricciones indicadas.

Pensamos que el cuello de botella en Galois está en este motor en tiempo de ejecución, ya que implementa en software
y en tiempo de ejecución un sistema que lleve cuenta de posibles condiciones de carrera o conflictos entre hilos, y
una serie de cerrojos de exclusión mutua y sistemas de recuperación que permitan que la ejecución paralela sea segura.
Tradicionalmente se ha visto que este tipo de sistemas tienen una gran penalización en el rendimiento. Queremos saber
cuán grande es esta penalización, y si puede ser mejorada haciendo uso de las nuevas instrucciones TSX de memoria
transaccional hardware que implementan los procesadores de Intel dirigidos a consumidores a partir de la 6ª generación.

### El "Tao" de la programación paralela
El proyecto Galois es la materialización de una investigación realizada por algunos de los mismos autores en los que
se proponen convertir la computación paralela en una ciencia. En el paper que se generó como resultado de esa
investigación [@pingali_tao_2011] se realiza una categorización de los algoritmos en base a tres propiedades, y la conclusión
que sacan es que se puede unificar la técnica para desarrollar algoritmos paralelos decidendo a posteriori si la
paralelización se puede realizar en tiempo de compilación o en ejecución, y de qué manera; pensando antes en la
estructura de los datos que en el algoritmo que se quiere hacer sobre ellos.

Las propiedades de este "Tao", en base a las que se analizan los algoritmos para categorizar su forma de paralelismo, son:

- **T**opología: La topología se enfoca en el análisis de la estructura sobre la que se centra la ejecución. En base
  a esta propiedad se categorizan las estructuras según su complejidad de Kolmogorov [@kolmogorov_tables_1998], que
  está relacionada con la dificultad que supone describirlas.
- Nodos **a**ctivos: Descripción de cómo un nodo se convierte en activo y del orden en que deben procesarse. Puede
  que el algoritmo exija un orden o no, y puede que los nodos se activen en función de la topología o en función de
  la información que contienen.
- **O**perador: Los operadores se categorizan en función de cómo modifican el grafo. Pueden modificarlo añadiendo o borrando
  vértices y aristas o actualizando valores, puede actualizar valores del grafo pero sin cambiar la topología, o simplemente
  puede no modificarla.

En base a esta categorización, algunas de las estructuras que define Galois [@_galois_docs] enfocadas a paralelizar estructuras irregulares son las siguientes. Todas ellas
pueden ser compartidas entre hilos con seguridad:

- `Galois::Graph::FirstGraph`: Un grafo que permite la mutación concurrente del mismo.
- `Galois::Graph::LC_CSR_Graph`: Un grafo que no permite ser mutado. Está optimizado para cargarse en memoria y ser muy rápido.
- `Galois::Bag`: Una generalización de un conjunto que permite elementos repetidos, pero sin orden.

También, se definen entre otros los siguientes operadores:

- Iterador para conjuntos desordenados: Itera sobre un conjunto sin garantías sobre el orden en el que sus elementos serán
  procesados. Permite que una iteración añada nuevos elementos al conjunto durante la ejecución. Acepta metadatos que
  indican condiciones sobre el paralelismo que se puede aplicar en la ejecución de las iteraciones del bucle.
- Iterador para conjuntos ordenados: Similar al anterior, pero además exige
  que siempre se procese el elemento mínimo del set primero. Se puede pasar un parámetro opcional a este operador que indica
  la política a seguir. Algunos ejemplos son cola de prioridad, FIFO, LIFO... 

Estas estructuras y operadores imponen restricciones y aportan metadatos al framework sobre cómo deben ser procesados estos
datos cuando se ejecutan de forma concurrente, estableciendo limitaciones que aseguran la corrección del software.

### Trabajo futuro en Galois
Actualmente, podemos ver que Galois está usando un acercamiento similar a la memoria transaccional por software para
alcanzar sus objetivos, manteniendo un histórico de operaciones que el motor en tiempo de ejecución comprueba
rutinariamente en busca de inconsistencias. Creemos que esta forma de atacar el problema es un cuello de botella que
se puede resolver sin cambios muy graves con memoria transaccional por hardware.

Si conseguimos introducir instrucciones RTM en el motor en tiempo de ejecución de Galois de forma que use las extensiones
TSX en lugar de su algoritmo software de detección de conflictos, podríamos obtener un paradigma de programación paralela
realmente simple que mantenga la seguridad y fiabilidad de los datos pero con una penalización de rendimiento mucho menor
que la de los sistemas de paralelización basados en software que se usan a día de hoy.

### Trabajo previo sobre memoria transaccional por hardware en Galois
Aunque no es claro a simple vista, analizando el código de Galois se puede ver que ya ellos han realizado algún
trabajo previo sobre memoria transaccional hardware, pero no hemos encontrado más referencias a este respecto en
la documentación del software o en los papers que han publicado los autores, salvo dos escuetas referencias:

En [@kulkarni_using_2006] encontramos la siguiente referencia:

> (...) These characteristics make simple hardware transactional
  memories (7) unsuitable for our purposes. However,
  software transactional memories, such as (5, 6), or
  more advanced hardware approaches (1, 12) may suffice,
  although their efficiency in the context of long-running
  transactions must still be studied (...) 

> (...) Estas características hacen que las memorias transaccionales
  hardware simples no sean apropiadas para nuestros objetivos.
  Sin embargo, las memorias transaccionales software o 
  aproximaciones hardware más avancadas podrían ser suficientes,
  aunque su eficiencia en el contexto de transacciones ejecutadas
  durante largo tiempo aún debe ser estudiada (...)

Y en [@kulkarni_lonestar:_2009], la siguiente:

> (...) It presents important workload characteristics of these programs.
  One key finding is that the transactions can be quite
  large, making them less suitable for hardware-based approaches. (...) 

> (...) Esto presenta importantes características de la carga de trabajo
  de estos programas. Un hallazgo clave es que las transacciones
  pueden ser bastante grandes, haciéndolas menos apropiadas para
  aproximaciones basadas en hardware. (...)

Para tratar de averiguar qué investigación concreta han llevado a cabo los autores de Galois sobre memoria transaccional,
tenemos que explorar el código fuente del proyecto. Lo primero que llama la atención es la bandera
`GALOIS_USE_HTM` que podemos configurar al compilar el proyecto, suponiendo que `HTM` significa _hardware transactional 
memory_. Los cambios que esta bandera realiza en el código es una implementación distinta de la clase `LoopStatistics`,
que si no se configura esta bandera, queda definida pero vacía. Al usar la bandera, esta clase recoge los motivos de
fallo de las transacciones de forma que se pueda analizar el rendimiento de las ejecuciones con memoria transaccional
hardware.

Otro hallazgo esclarecedor es que en la configuración de _CMake_ para construir el proyecto se indica una restricción de
forma que si se quiere compilar con el flag `GALOIS_USE_HTM`, es necesario que el compilador tenga el identificador `XL`.
Este identificador está haciendo referencia al compilador _IBM XL C++_ [@_cmake], diseñado para optimizar el código para
las arquitecturas IBM [@_web]. Esto tiene sentido, pues algunos procesadores IBM como la serie _POWER8_ o los _BlueGene/Q_ cuentan con
capacidades de memoria transaccional por hardware, aunque conceptualmente distintas a las que proporcionan las extensiones
TSX de Intel.

### Conclusiones sobre memoria transaccional por hardware en Galois
Dada la falta de publicaciones o información sobre este tema, y basándonos en la poca que hemos encontrado, podemos suponer
que los autores no encontraron suficiente el rendimiento de la memoria transaccional en procesadores de IBM como para continuar
la investigación por esa línea. Sin embargo, no podemos suponer que las prestaciones y herramientas que proporcionan las instrucciones
TSX de Intel repliquen ese rendimiento deficitario.

Aunque esta investigación quedaría fuera del alcance de este trabajo, sí sería interesante ver la complejidad que supondría
introducir las instrucciones TSX en el motor de tiempo de ejecución de Galois, evitando los bloqueos que pudieran producir
los cerrojos y la penalización en el rendimiento que supone el sistema de recuperación similar a la memoria transaccional
por sofware que forman el núcleo de Galois.
