# Galois

+ [Project home](http://iss.ices.utexas.edu/?p=projects/galois)

## Next steps

> You can ask the student to investigate the part of codes that handle the conflict detection and rollback.
  We want to replace that part with RTM. If you have any question, you can also directly ask Albert (see below)
  and cc me.

> as far as I know, all the contentions are managed in a .cpp file called Context.cpp. This file defines the check
  point where threads needs to roll back to when encountering a conflict with another thread. Besides src/Context.cpp,
  there are some helper functions defined in include/Galois/Runtime/Context.h . The conflict manager is primarily used in
  include/Galois/Runtime/DeterministicWork.h and include/Galois/Runtime/ParallelWork.h. So you can ask him/her to look at
  these files to figure out a way to apply TM.

### `src/Context.cpp`
Headers:
Galois/Runtime/Context.h
Galois/Runtime/MethodFlags.h
Galois/Runtime/ll/SimpleLock.h
Galois/Runtime/ll/CacheLineStorage.h

### `include/Galois/Runtime/Context.h`
Define SimpleRuntimeContext que gestiona los locks. Esta clase hereda de LockManagerBase. Quizás se puede hacer una nueva
clase que herede de ahí para implementar TSX, o modificando LockManagerBase.

LockManagerBase hereda de boost::noncopyable. Tiene funciones para gestionar el lock de un objeto Lockable.

SimpleRuntimeContext expone los metodos de LockManagerBase y tiene un registro de los locks que mantiene el contexto.

Veo que estas clases están definidas de una forma si se definió GALOIS_USE_SEQ_ONLY o no. Suponiendo que ese flag significa
que la ejecución de Galois será secuencial, quizás se pueda añadir otro flag GALOIS_USE_TSX que añada una tercera forma
de compilación a todos los bloques que comprueben ese flag permitiendo la implementación TSX.

GALOIS_USE_SEQ_ONLY está en:
- CMakeLists.txt:164
- include/Galois/config.h.in:23
- include/Galois/Runtime/Context.h:92
- include/Galois/Runtime/Context.h:223
- src/Context.cpp:116


En CMakeLists.txt se define una parte con constantes que habilitan características experimentales (USE_EXP). Una de ellas
es USE_HTM, que permite usar memoria transaccional hardware (línea 160). Para permitir usarlo, comprueba que CMAKE_CXX_COMPILER_ID MATCHES "XL"
y después activa los flags CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -qtm -qsmp=speculative", GALOIS_USE_HTM,
GALOIS_USE_SEQ_ONLY y GALOIS_USE_LONGJMP.

El flag GALOIS_USE_HTM se encuentra en include/Galois/Runtime/MethodFlags.h:40 y en include/Galois/Runtime/ParallelWork.h.

En ParallelWork.h define el iterador foreach de Galois. Al usar HTM, incluye el fichero speculation.h. Además, define la clase
LoopStatistics de forma especializada para HTM. Se ven varios usos de #pragma tm_atomic.



config.h
Runtime/ll/PtrLock.h
Runtime/ll/gio.h

### `include/Galois/Runtime/DeterministicWork.h`

### `include/Galois/Runtime/ParallelWork.h`

### Notas
Muchos ficheros tienen instrucciones de precompilador con diferentes bloques de código según si la constante GALOIS_USE_LONGJMP
es true o no; tambien si la constante GALOIS_USE_SEQ_ONLY está definida o no.

Si GALOIS_USE_LONGJMP, #include <setjmp.h>

### HTM compilation

We need to build Galois with USE_EXP and USE_HTM flags. I executed the next command on the /build/releasehtm directory:

```
cmake -DCMAKE_CXX_COMPILER=g++ -DCMAKE_C_COMPILER=gcc -DUSE_EXP=ON -DUSE_HTM=ON ../..
```

The build system stated that "Hardware transactional memory not supported". Looking at the line of the build script that
triggered the error, I see it is expecting that `CMAKE_CXX_COMPILER_ID MATCHES "XL"`. That variable represents a short
string with the name of the compiler being used [3]. The string `XL` is making reference to the IBM XL C++ compiler,
which optimizes to their architectures [4], so the use of HTM by Galois that I have been able to know despite the
non-existing documentation about it is outside of the scope of our work.


[1] http://iss.ices.utexas.edu/Publications/Papers/WTW2006.pdf
[2] http://iss.ices.utexas.edu/Publications/Papers/ispass2009.pdf
[3] https://cmake.org/cmake/help/v2.8.10/cmake.html#variable:CMAKE_LANG_COMPILER_ID
[4] http://www.ibm.com/developerworks/aix/library/au-aix-ibm-xl-compiler-built-in-functions/index.html
